package imcacat.adxvctl.util;

import java.io.UnsupportedEncodingException;

import org.junit.Test;
import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertTrue;

/**
 * I test <code>StringUtilities</code>.
 */
public class StringUtilitiesTest {
  @Test
  public void testEndsWithIgnoreCase01() {
    assertTrue(StringUtilities.endsWithIgnoreCase("image.cbf", ".cbf"));
  }

  @Test
  public void testEndsWithIgnoreCase02() {
    assertTrue(StringUtilities.endsWithIgnoreCase("image.CBF", ".cbf"));
  }

  @Test
  public void testEndsWithIgnoreCase03() {
    assertTrue(StringUtilities.endsWithIgnoreCase("image.cbf", ".CBF"));
  }

  @Test
  public void testEndsWithIgnoreCase04() {
    assertFalse(StringUtilities.endsWithIgnoreCase("image.cbf", ".cbx"));
  }

  @Test
  public void testEndsWithIgnoreCase05() {
    assertFalse(StringUtilities.endsWithIgnoreCase("i.cbf", "xi.cbf"));
  }

  @Test
  public void testEndsWithIgnoreCase06() {
    assertFalse(StringUtilities.endsWithIgnoreCase("image.cbf", ".cb"));
  }

  @Test
  public void testConvertNulTerminatedByteArrayToString01()
  throws UnsupportedEncodingException {
    String expected = "abc";
    String charsetName = "US-ASCII";
    assertEquals(expected,
      StringUtilities.convertNulTerminatedByteArrayToString(
        (expected + "\0").getBytes(charsetName), charsetName));
  }

  @Test
  public void testConvertNulTerminatedByteArrayToString02()
  throws UnsupportedEncodingException {
    String expected = "abc";
    byte[] input = new byte[] { 'a', 'b', 'c', '\0', 'd' };
    String charsetName = "US-ASCII";
    assertEquals(expected,
      StringUtilities.convertNulTerminatedByteArrayToString(input,
        charsetName));
  }

  @Test
  public void testConvertNulTerminatedByteArrayToString03()
  throws UnsupportedEncodingException {
    String expected = "a";
    byte[] input = new byte[] { 'a', '\0', 'b' };
    String charsetName = "US-ASCII";
    assertEquals(expected,
      StringUtilities.convertNulTerminatedByteArrayToString(input,
        charsetName));
  }

  @Test
  public void testConvertNulTerminatedByteArrayToString04()
  throws UnsupportedEncodingException {
    String expected = "";
    byte[] input = new byte[] { '\0', 'a' };
    String charsetName = "US-ASCII";
    assertEquals(expected,
      StringUtilities.convertNulTerminatedByteArrayToString(input,
        charsetName));
  }

  @Test
  public void testConvertNulTerminatedByteArrayToString05()
  throws UnsupportedEncodingException {
    String expected = "abc";
    byte[] input = new byte[] { 'a', 'b', 'c' };
    String charsetName = "US-ASCII";
    assertEquals(expected,
      StringUtilities.convertNulTerminatedByteArrayToString(input,
        charsetName));
  }
}
