package imcacat.adxvctl;

import java.awt.Container;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Frame;

import javax.swing.Icon;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.SwingConstants;

import com.jgoodies.forms.builder.PanelBuilder;
import com.jgoodies.forms.factories.Borders;
import com.jgoodies.forms.layout.CellConstraints;
import com.jgoodies.forms.layout.FormLayout;

/**
 * I am a standard About dialog displaying an application icon, an application
 * name, and an application version.
 * <p>
 * I call <code>pack()</code> in my constructor and adjust my size so callers
 * should not call <code>pack()</code> on me after creating me, otherwise my
 * size adjustments will be lost.
 */
public class AboutDialog extends JDialog {
  private static final long serialVersionUID = 1;

  protected String applicationName;
  protected String version;
  protected Icon applicationIcon;

  public AboutDialog(String applicationName, String version,
      Icon applicationIcon) {
    this(null, applicationName, version, applicationIcon);
  }

  public AboutDialog(Frame owner, String applicationName, String version,
      Icon applicationIcon) {
    super(owner);
    this.applicationName = applicationName;
    this.version = version;
    this.applicationIcon = applicationIcon;
    setContentPane(buildContentPane());
    pack();
    setSize(new Dimension(Math.max(getSize().width, 284), getSize().height));
    setResizable(false);
  }

  protected Container buildContentPane() {
    FormLayout layout = new FormLayout(
      // Columns
      "fill:pref:grow",
      // Rows
      "pref, 4dlu, pref, 4dlu, pref"
    );
    PanelBuilder builder = new PanelBuilder(layout);
    builder.border(Borders.DIALOG);
    CellConstraints cc = new CellConstraints();

    JLabel applicationIconLabel = new JLabel(applicationIcon);
    builder.add(applicationIconLabel, cc.xy(1, 1));

    JLabel applicationNameLabel = new JLabel(applicationName);
    applicationNameLabel.setHorizontalAlignment(SwingConstants.CENTER);
    applicationNameLabel.setFont(applicationNameLabel.getFont().deriveFont(
      Font.BOLD));
    builder.add(applicationNameLabel, cc.xy(1, 3));

    JLabel versionLabel = new JLabel("Version " + version);
    versionLabel.setHorizontalAlignment(SwingConstants.CENTER);
    builder.add(versionLabel, cc.xy(1, 5));

    return builder.getPanel();
  }
}
