package imcacat.adxvctl;

import imcacat.adxvctl.ComponentFactory;
import imcacat.adxvctl.Resources;
import imcacat.adxvctl.util.Pair;
import imcacat.epicsca4j.ca.EpicsCaException;
import imcacat.epicsca4j.util.StringUtilities;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.event.KeyEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.File;

import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.JFormattedTextField;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JSpinner;
import javax.swing.JToolBar;
import javax.swing.SpinnerNumberModel;
import javax.swing.WindowConstants;
import javax.swing.event.CaretEvent;
import javax.swing.event.CaretListener;
import javax.swing.filechooser.FileFilter;

import com.jgoodies.binding.adapter.SpinnerToValueModelConnector;
import com.jgoodies.forms.builder.PanelBuilder;
import com.jgoodies.forms.factories.Borders;
import com.jgoodies.forms.layout.CellConstraints;
import com.jgoodies.forms.layout.FormLayout;

import net.roydesign.app.Application;

/**
 * I am the main frame (i.e. window) for this application.
 */
public class AdxvctlFrame extends JFrame {
  private static final long serialVersionUID = 1;

  protected AdxvctlPresentationModel model;

  protected JDialog aboutDialog;
  protected JFormattedTextField collectedImageTextField;
  protected JComboBox imagePathComboBox;
  protected JSpinner updateEveryIntervalSpinner;
  protected SpinnerToValueModelConnector updateEveryIntervalSpinnerConnector;
  protected JButton updateNowButton;
  protected PropertyChangeListener updateEveryEnabledChangeListener;
  protected PropertyChangeListener collectedImageChangeListener;
  protected PropertyChangeListener userMessageChangeListener;
  protected ActionListener openActionListener;
  protected ActionListener aboutActionListener;

  protected File lastOpenedFile;

  public AdxvctlFrame(AdxvctlPresentationModel model)
  throws InterruptedException, EpicsCaException {
    super();
    this.model = model;

    initializeSpecialMenuBar();
    initializeFrame();
    initializeModelListening();
  }

  public void release() {
    model.getOpenAction().removeActionListener(openActionListener);
    model.getAboutAction().removeActionListener(aboutActionListener);
    model.getModel(
      Adxvctl.PROPNAME_UPDATE_EVERY_ENABLED).removeValueChangeListener(
        updateEveryEnabledChangeListener);
    updateEveryIntervalSpinnerConnector.release();
    model.getModel(
      Adxvctl.PROPNAME_COLLECTED_IMAGE).removeValueChangeListener(
        collectedImageChangeListener);
    model.getModel(Adxvctl.PROPNAME_USER_MESSAGE).removeValueChangeListener(
        userMessageChangeListener);
  }

  public void requestPreferredInitialFocus() {
    imagePathComboBox.requestFocusInWindow();
  }

  protected JMenuBar buildMenuBar() {
    JMenuBar menuBar = new JMenuBar();

    JMenu fileMenu = new JMenu("File");
    fileMenu.setMnemonic(KeyEvent.VK_F);
    JMenuItem fileOpenMenuItem = new JMenuItem(model.getOpenAction());
    fileOpenMenuItem.setMnemonic(KeyEvent.VK_O);
    fileMenu.add(fileOpenMenuItem);
    fileMenu.addSeparator();
    JMenuItem fileQuitMenuItem = new JMenuItem(model.getQuitAction());
    if (!Resources.isPlatformWindows())
      fileQuitMenuItem.setMnemonic(KeyEvent.VK_Q);
    fileMenu.add(fileQuitMenuItem);
    menuBar.add(fileMenu);

    JMenu historyMenu = new JMenu("History");
    historyMenu.setMnemonic(KeyEvent.VK_S);
    JMenuItem historyBackMenuItem = new JMenuItem(model.getBackAction());
    historyBackMenuItem.setMnemonic(KeyEvent.VK_B);
    historyMenu.add(historyBackMenuItem);
    JMenuItem historyForwardMenuItem =
      new JMenuItem(model.getForwardAction());
    historyForwardMenuItem.setMnemonic(KeyEvent.VK_F);
    historyMenu.add(historyForwardMenuItem);
    historyMenu.addSeparator();
    JMenuItem historyClearMenuItem = new JMenuItem(model.getClearAction());
    historyClearMenuItem.setMnemonic(KeyEvent.VK_C);
    historyMenu.add(historyClearMenuItem);
    menuBar.add(historyMenu);

    JMenu helpMenu = new JMenu("Help");
    helpMenu.setMnemonic(KeyEvent.VK_H);
    JMenuItem helpAboutMenuItem = new JMenuItem(model.getAboutAction());
    helpAboutMenuItem.setMnemonic(KeyEvent.VK_A);
    helpMenu.add(helpAboutMenuItem);
    menuBar.add(helpMenu);

    return menuBar;
  }

  /* If running on Mac OS X, this makes the special Mac OS X Application
   * About and Quit menu items work as expected.
   */
  protected void initializeSpecialMenuBar() {
    Application mrjApp = Application.getInstance();
    mrjApp.getAboutJMenuItem().addActionListener(model.getAboutAction());
    mrjApp.getQuitJMenuItem().addActionListener(model.getQuitAction());
  }

  protected void initializeFrame()
  throws InterruptedException, EpicsCaException {
    setTitle(Resources.getApplicationName());
    setIconImage(Resources.getAppIcon().getImage());
    setJMenuBar(buildMenuBar());
    setContentPane(buildContentPane());
    setDefaultCloseOperation(WindowConstants.DO_NOTHING_ON_CLOSE);

    addWindowListener(new WindowAdapter() {
      public void windowClosing(WindowEvent event) {
        model.getQuitAction().actionPerformed(
          new ActionEvent(this, ActionEvent.ACTION_PERFORMED, null));
      }
    });

    /* WORKAROUND: Change caret position by one after a resize so that my
     * custom CaretListener will reset the position to force the text to
     * scroll right.  Without this, when the text is longer than the text
     * field and the window is resized to make the text fit in the text field
     * and then resized back down so that the text does not fit, the text will
     * stay scrolled left. */
    addComponentListener(new ComponentAdapter() {
      public void componentResized(ComponentEvent event) {
        if (collectedImageTextField.isFocusOwner()) return;
        String text = collectedImageTextField.getText();
        if (text == null) return;
        collectedImageTextField.setCaretPosition(
          Math.max(0, text.length() - 1));
      }
    });
  }

  protected JComponent buildContentPane() {
    FormLayout layout = new FormLayout(
      // Columns
      "right:default, 4dlu, left:default, 4dlu, left:default, 2dlu, " +
        "fill:max(30dlu;default), 2dlu, fill:max(45dlu;default):grow, " +
        "2dlu, left:pref",
      // Rows
      "pref, 4dlu, pref, 2dlu, pref"
    );
    layout.setRowGroups(new int[][] {
      // Fields
      {1, 3}
    });
    PanelBuilder builder = new PanelBuilder(layout);
    builder.border(Borders.DLU4);
    CellConstraints cc = new CellConstraints();

    builder.addLabel("Collected:", cc.xy(1, 1));
    collectedImageTextField = ComponentFactory.createFormattedTextField(
      model.getModel(Adxvctl.PROPNAME_COLLECTED_IMAGE), new FileFormatter());
    collectedImageTextField.setEditable(false);
    collectedImageTextField.setFocusable(false);
    /* WORKAROUND: Force the text to scroll right to display the end of the
     * file path rather than the beginning of the file path when the entire
     * path does not fit in the text field. */
    collectedImageTextField.addCaretListener(new CaretListener() {
      public void caretUpdate(CaretEvent event) {
        if (collectedImageTextField.isFocusOwner()) return;
        int textLength = collectedImageTextField.getText().length();
        if (textLength == event.getDot()) return;
        collectedImageTextField.setCaretPosition(textLength);
      }
    });
    builder.add(collectedImageTextField, cc.xyw(3, 1, 9));

    builder.addLabel("Adxv:", cc.xy(1, 3));
    final FileCellRenderer cellRenderer =
      new FileCellRenderer(FileCellRenderer.FLAG_LEFT_TRUNCATE);
    imagePathComboBox = ComponentFactory.createComboBox(
      model.getImageSelectionInList(), cellRenderer);
    imagePathComboBox.addComponentListener(new ComponentAdapter() {
      public void componentResized(ComponentEvent event) {
        cellRenderer.setListWidth(imagePathComboBox.getWidth());
      }
    });
    imagePathComboBox.addItemListener(new ItemListener() {
      public void itemStateChanged(ItemEvent e) {
        cellRenderer.setListWidth(imagePathComboBox.getWidth());
      }
    });
    builder.add(imagePathComboBox, cc.xyw(3, 3, 7));
    JToolBar imagePathNavigationBar = new JToolBar();
    imagePathNavigationBar.setFloatable(false);
    JButton backButton = imagePathNavigationBar.add(model.getBackAction());
    backButton.setText("");
    backButton.setIcon(Resources.getGoBackIcon());
    backButton.setToolTipText("Go to previous image");
    JButton forwardButton = imagePathNavigationBar.add(
      model.getForwardAction());
    forwardButton.setText("");
    forwardButton.setIcon(Resources.getGoForwardIcon());
    forwardButton.setToolTipText("Go to next image");
    builder.add(imagePathNavigationBar, cc.xy(11, 3));

    updateNowButton = new JButton(model.getUpdateNowAction());
    updateNowButton.setEnabled(false);
    builder.add(updateNowButton, cc.xy(3, 5));
    JCheckBox updateEveryEnabledCheckBox = ComponentFactory.createCheckBox(
      model.getModel(Adxvctl.PROPNAME_UPDATE_EVERY_ENABLED), "Update every");
    builder.add(updateEveryEnabledCheckBox, cc.xy(5, 5));
    Pair<JSpinner,SpinnerToValueModelConnector> spinnerPair =
      ComponentFactory.createSpinner(
      model.getModel(Adxvctl.PROPNAME_UPDATE_EVERY_INTERVAL),
      new SpinnerNumberModel(new Integer(1), new Integer(1), null,
        new Integer(1)));
    updateEveryIntervalSpinner = spinnerPair.getFirst();
    updateEveryIntervalSpinnerConnector = spinnerPair.getSecond();
    builder.add(updateEveryIntervalSpinner, cc.xy(7, 5));
    JLabel updateEveryImagesLabel = new JLabel("image(s)");
    builder.add(updateEveryImagesLabel, cc.xy(9, 5));

    return builder.getPanel();
  }

  protected void initializeModelListening() {
    openActionListener = new ActionListener() {
      public void actionPerformed(ActionEvent event) {
        openActionPerformed();
      }
    };
    model.getOpenAction().addActionListener(openActionListener);

    aboutActionListener = new ActionListener() {
      public void actionPerformed(ActionEvent event) {
        aboutActionPerformed();
      }
    };
    model.getAboutAction().addActionListener(aboutActionListener);

    updateEveryEnabledChangeListener = new PropertyChangeListener() {
      public void propertyChange(PropertyChangeEvent event) {
        updateEveryEnabledChanged((Boolean)event.getNewValue());
      }
    };
    model.getModel(
      Adxvctl.PROPNAME_UPDATE_EVERY_ENABLED).addValueChangeListener(
        updateEveryEnabledChangeListener);

    collectedImageChangeListener = new PropertyChangeListener() {
      public void propertyChange(PropertyChangeEvent event) {
        collectedImageChanged((File)event.getNewValue());
      }
    };
    model.getModel(Adxvctl.PROPNAME_COLLECTED_IMAGE).addValueChangeListener(
      collectedImageChangeListener);

    userMessageChangeListener = new PropertyChangeListener() {
      public void propertyChange(PropertyChangeEvent event) {
        userMessageChanged((String)event.getNewValue());
      }
    };
    model.getModel(Adxvctl.PROPNAME_USER_MESSAGE).addValueChangeListener(
      userMessageChangeListener);
  }

  protected void openActionPerformed() {
    JFileChooser chooser = new JFileChooser();
    chooser.setFileSelectionMode(JFileChooser.FILES_ONLY);
    chooser.setDialogTitle("Open");
    File file = lastOpenedFile;
    if (file == null || !file.getParentFile().exists())
      file = model.getSelectedImage();
    if (file != null && file.exists()) {
      chooser.setSelectedFile(file);
    } else if (file != null && file.getParentFile().exists()) {
        chooser.setCurrentDirectory(file.getParentFile());
    } else {
      chooser.setCurrentDirectory(null);
    }
    FileFilter imgFilter = new FileChooserFileFilter(
      new ExtensionFileFilter(".img"), "IMG images (.img)");
    FileFilter cbfFilter = new FileChooserFileFilter(
      new ExtensionFileFilter(".cbf"), "CBF images (.cbf)");
    chooser.addChoosableFileFilter(cbfFilter);
    chooser.addChoosableFileFilter(imgFilter);
    if (file != null && imgFilter.accept(file))
      chooser.setFileFilter(imgFilter);
    else
      chooser.setFileFilter(cbfFilter);
    int returnState = chooser.showDialog(this, "Open");
    if (returnState == JFileChooser.APPROVE_OPTION) {
      lastOpenedFile = chooser.getSelectedFile();
      model.getModel(Adxvctl.PROPNAME_UPDATE_EVERY_ENABLED).setValue(false);
      model.setSelectedImage(lastOpenedFile);
    }
  }

  protected void aboutActionPerformed() {
    if (aboutDialog == null) {
      aboutDialog = new AboutDialog(this, Resources.getApplicationName(),
        Resources.getApplicationVersion(), Resources.getLargeAppIcon());
      aboutDialog.setLocationRelativeTo(this);
    }
    aboutDialog.setVisible(true);
  }

  protected void updateEveryEnabledChanged(Boolean newValue) {
    updateEveryIntervalSpinner.setEnabled(
      newValue != null && newValue.booleanValue());
  }

  protected void collectedImageChanged(File newImage) {
    updateNowButton.setEnabled(newImage != null);
  }

  protected void userMessageChanged(String newMessage) {
    if (newMessage == null || newMessage.equals("")) return;
    String message = stripUserMessagePrefix(newMessage);
    int messageType = deriveUserMessageType(newMessage);
    JOptionPane.showMessageDialog(this,
      StringUtilities.wordWrap(message, 78, "\n"),
      "",
      messageType);
  }

  protected int deriveUserMessageType(String message) {
    if (message.startsWith(Adxvctl.USER_MESSAGE_PREFIX_WARN))
      return JOptionPane.WARNING_MESSAGE;
    else if (message.startsWith(Adxvctl.USER_MESSAGE_PREFIX_ERROR))
      return JOptionPane.ERROR_MESSAGE;
    else
      return JOptionPane.PLAIN_MESSAGE;
  }

  protected String stripUserMessagePrefix(String message) {
    if (message.startsWith(Adxvctl.USER_MESSAGE_PREFIX_WARN))
      return message.substring(Adxvctl.USER_MESSAGE_PREFIX_WARN.length());
    else if (message.startsWith(Adxvctl.USER_MESSAGE_PREFIX_ERROR))
      return message.substring(Adxvctl.USER_MESSAGE_PREFIX_ERROR.length());
    else
      return message;
  }

  protected String getApplicationVersion() {
    return Resources.getApplicationVersion();
  }
}
