package imcacat.adxvctl;

import imcacat.epicsca4j.ca.EpicsCaException;
import imcacat.epicsca4j.util.ObjectUtilities;

import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.File;
import java.util.List;

import javax.swing.Action;
import javax.swing.KeyStroke;
import javax.swing.ListModel;

import com.jgoodies.binding.PresentationModel;
import com.jgoodies.binding.list.SelectionInList;
import com.jgoodies.binding.value.ValueModel;
import com.jgoodies.common.collect.LinkedListModel;

/**
 * I am the presentation model for {@link AdxvctlFrame}.
 */
public class AdxvctlPresentationModel extends PresentationModel<Adxvctl> {
  private static final long serialVersionUID = 1;

  protected static final int MAX_IMAGE_LIST_SIZE = 1000;

  protected ListenableAction openAction;
  protected ListenableAction quitAction;
  protected ListenableAction backAction;
  protected ListenableAction forwardAction;
  protected ListenableAction updateNowAction;
  protected ListenableAction clearAction;
  protected ListenableAction aboutAction;

  protected SelectionInList<File> imageSelectionInList;
  protected boolean ignoreImageSelectionIndexChange = false;

  protected ValueModel updateEveryEnabledModel;

  public AdxvctlPresentationModel(Adxvctl bean)
  throws InterruptedException, EpicsCaException {
    super(bean);

    openAction = new ListenableAction("Open\u2026");
    openAction.putValue(Action.ACCELERATOR_KEY,
      KeyStroke.getKeyStroke(KeyEvent.VK_O,
        Toolkit.getDefaultToolkit().getMenuShortcutKeyMask()));

    if (Resources.isPlatformWindows()) {
      quitAction = new ListenableAction("Exit");
    } else {
      quitAction = new ListenableAction("Quit");
      quitAction.putValue(Action.ACCELERATOR_KEY,
        KeyStroke.getKeyStroke(KeyEvent.VK_Q,
          Toolkit.getDefaultToolkit().getMenuShortcutKeyMask()));
    }
    quitAction.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        quitActionPerformed();
      }
    });

    backAction = new ListenableAction("Back");
    backAction.putValue(Action.ACCELERATOR_KEY,
      KeyStroke.getKeyStroke(KeyEvent.VK_OPEN_BRACKET,
        Toolkit.getDefaultToolkit().getMenuShortcutKeyMask()));
    backAction.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        backActionPerformed();
      }
    });
    backAction.setEnabled(false);

    forwardAction = new ListenableAction("Forward");
    forwardAction.putValue(Action.ACCELERATOR_KEY,
      KeyStroke.getKeyStroke(KeyEvent.VK_CLOSE_BRACKET,
        Toolkit.getDefaultToolkit().getMenuShortcutKeyMask()));
    forwardAction.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        forwardActionPerformed();
      }
    });
    forwardAction.setEnabled(false);

    updateNowAction = new ListenableAction("Update Now");
    updateNowAction.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        updateNowActionPerformed();
      }
    });

    clearAction = new ListenableAction("Clear");
    clearAction.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        clearActionPerformed();
      }
    });
    clearAction.setEnabled(false);

    aboutAction = new ListenableAction("About " +
      Resources.getApplicationName());

    imageSelectionInList = new SelectionInList<File>(
        (ListModel)(new LinkedListModel<File>()));
    imageSelectionInList.addPropertyChangeListener(
      SelectionInList.PROPERTY_SELECTION_INDEX,
      new PropertyChangeListener() {
        public void propertyChange(PropertyChangeEvent event) {
          imageSelectionIndexChanged((Integer)event.getNewValue());
        }
      });

    getModel(Adxvctl.PROPNAME_ADXV_IMAGE).addValueChangeListener(
      new PropertyChangeListener() {
        public void propertyChange(PropertyChangeEvent event) {
          adxvImageChanged((File)event.getNewValue());
        }
      });

    updateEveryEnabledModel = getModel(Adxvctl.PROPNAME_UPDATE_EVERY_ENABLED);
  }

  public ListenableAction getOpenAction() {
    return openAction;
  }

  public ListenableAction getQuitAction() {
    return quitAction;
  }

  public ListenableAction getBackAction() {
    return backAction;
  }

  public ListenableAction getForwardAction() {
    return forwardAction;
  }

  public ListenableAction getUpdateNowAction() {
    return updateNowAction;
  }

  public ListenableAction getClearAction() {
    return clearAction;
  }

  public ListenableAction getAboutAction() {
    return aboutAction;
  }

  public SelectionInList<File> getImageSelectionInList() {
    return imageSelectionInList;
  }

  public File getSelectedImage() {
    return imageSelectionInList.getSelection();
  }

  public void setSelectedImage(File image) {
    if (image == null) {
      imageSelectionInList.clearSelection();
      return;
    }
    List<File> imageList = imageSelectionInList.getList();
    int imageIndex = imageList.indexOf(image);
    if (imageIndex < 0) {
      ignoreImageSelectionIndexChange = true;
      if (imageList.size() == MAX_IMAGE_LIST_SIZE)
        imageList.remove(imageList.size() - 1);
      imageList.add(0, image);
      imageSelectionInList.setSelectionIndex(0);
      ignoreImageSelectionIndexChange = false;
      imageSelectionIndexChanged(0);
    } else {
      imageSelectionInList.setSelectionIndex(imageIndex);
    }
  }

  protected void quitActionPerformed() {
    AdxvctlSystem.basicExit(0);
  }

  protected void backActionPerformed() {
    int index = imageSelectionInList.getSelectionIndex();
    if (index < 0 || index == imageSelectionInList.getSize() - 1) return;
    updateEveryEnabledModel.setValue(false);
    imageSelectionInList.setSelectionIndex(index + 1);
  }

  protected void forwardActionPerformed() {
    int index = imageSelectionInList.getSelectionIndex();
    if (index <= 0) return;
    updateEveryEnabledModel.setValue(false);
    imageSelectionInList.setSelectionIndex(index - 1);
  }

  protected void updateNowActionPerformed() {
    getBean().updateNow();
  }

  protected void clearActionPerformed() {
    imageSelectionInList.getList().clear();
  }

  protected void imageSelectionIndexChanged(int index) {
    if (ignoreImageSelectionIndexChange) return;
    clearAction.setEnabled(index >= 0);
    backAction.setEnabled(index >= 0 &&
      index != imageSelectionInList.getSize() - 1);
    forwardAction.setEnabled(index > 0);
    File selection =
      (index < 0) ? null : imageSelectionInList.getElementAt(index);
    File adxvImage = getBean().getAdxvImage();
    if (ObjectUtilities.areEqual(selection, adxvImage)) return;
    if (selection != null) updateEveryEnabledModel.setValue(false);
    getBean().setAdxvImage(selection);
  }

  protected void adxvImageChanged(File image) {
    setSelectedImage(image);
  }
}
