package imcacat.adxvctl;

import imcacat.epicsca4j.event.QueuedEventDispatcher;
import imcacat.epicsca4j.model.EpicsPv;

import javax.swing.Action;
import javax.swing.SwingUtilities;
import javax.swing.UIManager;
import javax.swing.UnsupportedLookAndFeelException;

import org.apache.log4j.LogManager;
import org.apache.log4j.Logger;

import com.jgoodies.looks.plastic.PlasticLookAndFeel;

/**
 * I am the core of the adxvctl application.
 *
 * I handle starting the adxvctl application as well as shutting it down.
 */
public class AdxvctlSystem {
  private static final Logger LOG = Logger.getLogger(AdxvctlSystem.class);

  protected static Action specialMenuBarQuitAction;
  protected static Action specialMenuBarAboutAction;

  public static void main(String[] args) {
    initializeUiLookAndFeel();
    initializeEpicsca4jEventDispatcher();
    initializeEpicsPv();
    SwingUtilities.invokeLater(new Runnable() {
      public void run() {
        try {
          AdxvctlFrame frame = new AdxvctlFrame(
            new AdxvctlPresentationModel(
              new Adxvctl(Resources.getImagePathPvNames())));
          frame.pack();
          frame.setMinimumSize(frame.getSize());
          frame.setLocationRelativeTo(null);
          frame.requestPreferredInitialFocus();
          frame.setVisible(true);
        } catch (Exception e) {
          LOG.fatal("Fatal exception", e);
          basicExit(1);
        }
      }});
  }

  public static void basicExit(int exitStatus) {
    LogManager.shutdown();
    System.exit(exitStatus);
  }

  protected static void initializeUiLookAndFeel() {
    PlasticLookAndFeel.setSelectTextOnKeyboardFocusGained(true);
    try {
      UIManager.setLookAndFeel(new PlasticLookAndFeel());
    } catch (UnsupportedLookAndFeelException e) {
      LOG.fatal("Failed to set Look & Feel", e);
      throw new RuntimeException(e);
    }
  }

  protected static void initializeEpicsca4jEventDispatcher() {
    imcacat.epicsca4j.util.Resources.getDefaultProperties().setProperty(
      imcacat.epicsca4j.util.Resources.
        EPICSCA4J_JCA_CONTEXT_EVENT_DISPATCHER_KEY,
      QueuedEventDispatcher.class.getCanonicalName());
  }

  protected static void initializeEpicsPv() {
    EpicsPv.setFirePropertyChangeEventsFromEdt(true);
  }
}
