package imcacat.adxvctl;

import java.io.UnsupportedEncodingException;

import org.apache.log4j.Logger;

import imcacat.adxvctl.util.StringUtilities;
import imcacat.epicsca4j.convert.BasicEpicsChannelAccessValueConverter;
import gov.aps.jca.dbr.DBR;

/**
 * I convert <code>byte</code> array values from EPICS Channel Access to
 * <code>String</code> values and vice-versa.  Any <code>String</code> value
 * must not contain the NUL character; if it does, the first NUL will be
 * considered the end of the string.
 */
public class ByteArrayToStringEpicsCaValueConverter extends
    BasicEpicsChannelAccessValueConverter {
  private static final Logger LOG = Logger.getLogger(
    ByteArrayToStringEpicsCaValueConverter.class);

  protected static final String CHARACTER_SET_NAME = "UTF-8";

  public Object convertToJava(DBR dbrValue, boolean isArray) {
    if (!isArray) throw new IllegalArgumentException("non-array unsupported");
    return convertToString((byte[])dbrValue.getValue());
  }

  public Object convertToChannelAccess(Object javaValue) {
    return convertToByteArray((String)javaValue);
  }

  protected String convertToString(byte[] value) {
    try {
      return StringUtilities.convertNulTerminatedByteArrayToString(value,
        CHARACTER_SET_NAME);
    } catch (UnsupportedEncodingException e) {
      LOG.error("Failed to decode byte array", e);
      return "";
    }
  }

  protected byte[] convertToByteArray(String value) {
    try {
      return (value + "\0").getBytes(CHARACTER_SET_NAME);
    } catch (UnsupportedEncodingException e) {
      LOG.error("Failed to encode byte array", e);
      return new byte[] {'\0'};
    }
  }
}
