package imcacat.adxvctl;

import imcacat.adxvctl.util.StringUtilities;

import java.io.File;
import java.io.FileFilter;

/**
 * I am a {@link java.io.FileFilter} that accepts <code>File</code> objects
 * that are files and whose names end with a specified "extension" string.
 * The extension string matching is case insensitive.
 */
public class ExtensionFileFilter implements FileFilter {
  protected String extension;

  /**
   * Constructs instance with specified extension.
   *
   * @param extension    extension to match; should include leading "." if
   *                     matching a typical extension (e.g. ".txt")
   */
  public ExtensionFileFilter(String extension) {
    super();
    this.extension = extension;
  }

  public boolean accept(File aFile) {
    return aFile.isFile() &&
      StringUtilities.endsWithIgnoreCase(aFile.getName(), extension);
  }
}
