package imcacat.adxvctl;

import imcacat.adxvctl.util.Version;

import java.io.File;
import java.net.URL;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;
import java.util.regex.Pattern;

import javax.swing.ImageIcon;

/**
 * I provide resources (e.g. icons and properties) for this application.
 */
public class Resources {
  protected static final Pattern IMAGE_PATH_PV_NAMES_DELIM =
    Pattern.compile(",");

  protected static File userHomeDirectory;
  protected static File adxvControlFile;

  private Resources() {
    super();
  }

  public static List<String> getImagePathPvNames() {
    return Collections.unmodifiableList(Arrays.asList(
      IMAGE_PATH_PV_NAMES_DELIM.split(System.getProperty("imagePathPvs"))));
  }

  public static ImageIcon getAppIcon() {
    String resourcePath = "resources/app-icon-16.png";
    URL url = Resources.class.getResource("/" + resourcePath);
    return (url == null) ? null : new ImageIcon(url);
  }

  public static ImageIcon getLargeAppIcon() {
    String resourcePath = "resources/app-icon-48.png";
    URL url = Resources.class.getResource("/" + resourcePath);
    return (url == null) ? null : new ImageIcon(url);
  }

  public static ImageIcon getGoBackIcon() {
    String resourcePath = "resources/go-previous.png";
    URL url = Resources.class.getResource("/" + resourcePath);
    return (url == null) ? null : new ImageIcon(url);
  }

  public static ImageIcon getGoForwardIcon() {
    String resourcePath = "resources/go-next.png";
    URL url = Resources.class.getResource("/" + resourcePath);
    return (url == null) ? null : new ImageIcon(url);
  }

  public static String getApplicationName() {
    return "Adxvctl";
  }

  public static String getApplicationLongName() {
    return "Adxv Control";
  }

  public static String getApplicationVersion() {
    return Version.getId();
  }

  public static boolean isPlatformMacOsX() {
    return System.getProperty("os.name").toLowerCase().startsWith("mac os x");
  }

  public static boolean isPlatformWindows() {
    return System.getProperty("os.name").toLowerCase().startsWith("windows");
  }

  public static synchronized File getUserHomeDirectory() {
    if (userHomeDirectory == null)
      userHomeDirectory = new File(System.getProperty("user.home"));
    return userHomeDirectory;
  }

  public static synchronized File getAdxvControlFile() {
    if (adxvControlFile == null)
      adxvControlFile = new File(getUserHomeDirectory(),
        ".adxvctl" + File.separator + "autoload");
    return adxvControlFile;
  }

  public static synchronized String getLineSeparator() {
    return System.getProperty("line.separator");
  }
}
